! =================================================================================================================================
! MODULE       : stomate_mark_kill
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF         Marks vegetation for killing by natural causes.  Replaces
!                lpj_kill.  Does not work with the DGVM.
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S)	:
!!
!! SVN          :
!! $HeadURL: svn://forge.ipsl.jussieu.fr/orchidee/branches/ORCHIDEE-DOFOCO/ORCHIDEE/src_stomate/stomate_mark_kill.f90 $
!! $Date: 2013-09-13 17:14:52 +0200 (Fri, 13 Sep 2013) $
!! $Revision: 1470 $
!! \n
!_ ================================================================================================================================


MODULE stomate_mark_kill

  ! modules used:

  USE ioipsl_para
  USE stomate_data
  USE pft_parameters
  USE constantes
  USE function_library,       ONLY : distribute_mortality_biomass_yyt,&
                                     nmax,  wood_to_dia,  P50_vary_class

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC kill_prognostic

CONTAINS

!! ================================================================================================================================
!! SUBROUTINE  : kill_prognostic
!!
!>\BRIEF       Kills natural pfts that have low biomass or low number of individuals.
!!             Does not change the biomass or litter pools.
!!
!! DESCRIPTION : Kills natural PFTS.  Forests can die through self-thinning or
!!             environmental conditions.  Crops can die through environmental
!!             conditions, and grasses don't die at all.  The total number
!!             of individuals to be killed in each circ class are outputted
!!             in circ_class_kill, and the biomass is actually killed in
!!             gap_prognostic.
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): ::circ_class_kill
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE kill_prognostic (npts, whichroutine, lm_lastyearmax, ind, &
       PFTpresent, biomass, npp_longterm, circ_class_biomass, &
       circ_class_n, circ_class_kill, rue_longterm, turnover_longterm, &
       dt, veget_max, st_dist, forest_managed,lpft_replant,stem_plc,psi_stem,counter_psi,counter_2,counter_0,st_mortality,d_mortality,kill_st,kill_d,circ_class_st, circ_class_mor, circ_class_litter, circ_class_leaflitter, circ_class_saplitter, circ_class_heartlitter)

 !! 0. Variable and parameter description

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                                :: npts                !! Domain size (unitless)
    CHARACTER(LEN=10), INTENT(in)                             :: whichroutine        !! Message (unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)              :: veget_max           !! "maximal" coverage fraction of a PFT on 
                                                                                     !! the ground (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)              :: lm_lastyearmax      !! Last year's maximum leaf mass, for each 
                                                                                     !! PFT @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)              :: rue_longterm        !! Longterm radiation use efficiency 
                                                                                     !! (??units??)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                                   INTENT(in) :: turnover_longterm   !! "Long term" (default 3-year) turnover 
                                                                                     !! rate @tex $(gC m^{-2} year^{-1})$ @endtex
    REAL(r_std), INTENT(in)                                   :: dt                  !! Time step (days)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)              :: npp_longterm        !! "Long term" (default = 3-year) net primary 
                                                                                     !! productivity 
                                                                                     !! @tex $(gC.m^{-2} year^{-1})$ @endtex
    REAL(r_std), DIMENSION(:), INTENT(in)                     :: st_dist             !! The probability distribution of trees 
                                                                                     !! removed from a circ class in case of  
                                                                                     !! self thinning (unitless).
    INTEGER(i_std), DIMENSION (:,:), INTENT(in)               :: forest_managed      !! forest management flag (is the forest 
                                                                                     !! being managed?)

    !! 0.2 Output variables


    !! 0.3 Modified variables

    LOGICAL, DIMENSION(npts,nvm), INTENT(inout)               :: PFTpresent          !! Is pft there (true/false)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)           :: ind                 !! Number of individuals 
                                                                                     !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                                INTENT(inout) :: biomass             !! Biomass @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), &                         
                                                INTENT(inout) :: circ_class_biomass  !! Biomass of the componets of the model  
                                                                                     !! tree within a circumference
                                                                                     !! class @tex $(gC ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(inout)     :: circ_class_n        !! Number of individuals in each circ class
                                                                                     !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nfm_types,ncut_times),&
                                            INTENT(inout)     :: circ_class_kill     !! Number of trees within a circ that needs
                                                                                     !! to be killed @tex $(ind m^{-2})$ @endtex
    LOGICAL, DIMENSION(npts,nvm), INTENT(inout)               :: lpft_replant        !! Set to true if a PFT has been clearcut
                                                                                     !! and needs to be replaced by another species

    !! 0.4 Local variables

    INTEGER(i_std)                                            :: ipts, ivm,icir,ipar !! Indices       (unitless)
    INTEGER(i_std)                                            :: iele, imbc, istep   !! Indices       (unitless)
    INTEGER(i_std)                                            :: ifm                 !! Indices       (unitless)
    LOGICAL, DIMENSION(npts)                                  :: was_killed          !! Bookkeeping   (true/false)
    REAL(r_std), DIMENSION(ncirc)                             :: diameters_temp      !! Trunk diameter calculated from allometry
    REAL(r_std)                                               :: Dg                  !! Quadratic mean diameter of the stand (cm)
    REAL(r_std)                                               :: bm_difference       !! the difference between self-thinning biomass
                                                                                     !! and that due to environmental mortality
    REAL(r_std)                                               :: difference          !! the difference between the self-thinning 
                                                                                     !! density
                                                                                     !! and the true density (individuals per m**2)
    REAL(r_std), DIMENSION(npts,nvm)                          :: st_dens             !! the self-thinning density of the stand 
                                                                                     !! (individuals per m**2)
    REAL(r_std), DIMENSION(npts,nvm),INTENT(out)              :: st_mortality        !! biomass killed due to self-thinning density 
    REAL(r_std), DIMENSION(npts,nvm),INTENT(out)              :: d_mortality         !! biomass killed due to the environment
    REAL(r_std)                                               :: delta_biomass       !! Net biomass increase for the previous
                                                                                     !! year @tex $(gC m^{-2} year^{-1})$ @endtex
    REAL(r_std)                                               :: vigour              !! Growth efficiency, an indicator of tree
                                                                                     !! vitality, used to calculate mortality
    REAL(r_std)                                               :: mortality_greff     !! Mortality rate derived by growth
	                                                                             !! efficiency @tex $(year^{-1})$ @endtex
    REAL(r_std)                                               :: mortality           !! Mortality (fraction of trees that is
                                                                                     !! dying per time step)
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements)        :: check_intern        !! Contains the components of the internal
                                                                                     !! mass balance chech for this routine
                                                                                     !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                :: closure_intern      !! Check closure of internal mass balance
                                                                                     !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                :: pool_start, pool_end!! Start and end pool of this routine 
                                                                                     !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std)                                               :: temp  
    LOGICAL                                                   :: lconverged          !! Checking for loop convergence
    REAL(r_std), DIMENSION(ncirc)                             :: target_st_kill      !! How many trees from each circumference
                                                                                     !! class we would like to kill.
                                                                                     !! @tex $(trees m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                             :: true_st_kill        !! How many trees from each circumference
                                                                                     !! class that we actually killed.
                                                                                     !! @tex $(trees m^{-2})$ @endtex
    REAL(r_std)                                               :: excess_ind          !! THe difference between the amount of
                                                                                     !! trees we want to kill and how many we
                                                                                     !! can actually kill based on our
                                                                                     !! circ_class distribution.
                                                                                     !! @tex $(trees m^{-2})$ @endtex
    ! added by Yitong Yao in 06 Jan 2020 16:06
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(in)      :: stem_plc
    ! added by Yitong Yao in 06 Jan 2020 16:06 above line
   
    ! added by Yitong Yao in 06 Jan 2020 16:06
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(in)      :: psi_stem
    ! added by Yitong Yao in 06 Jan 2020 16:06 above line

    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)   :: kill_st
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)   :: kill_d
    
    
    REAL(r_std), DIMENSION(ncirc)                             :: circ_diameters_temp     
    REAL(r_std), DIMENSION(ncirc)                             :: circ_class_P50
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(inout)              :: counter_psi
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(inout)              :: counter_2
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(inout)              :: counter_0



    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(out)                     :: circ_class_mor 
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(out)       :: circ_class_st 
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(out)       :: circ_class_litter
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(out)       :: circ_class_leaflitter
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(out)       :: circ_class_saplitter
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(out)       :: circ_class_heartlitter
    !! 
        REAL(r_std) :: yyt_test
    !!  
!    INTEGER,SAVE :: counter=0
!_ ================================================================================================================================
   
    !++++ TEMP +++++
 !   counter=counter+1
    !++++++++++



   IF (bavard.GE.2) WRITE(numout,*) 'Entering kill_prognostic'

 !! 1. Initialize 
   !! 1.1 Initialize check for mass balance closure
   !  The mass balance is calculated at the end of this routine
   !  in section 4.
   pool_start = zero
   DO ipar = 1,nparts
      DO iele = 1,nelements
         !  Initial biomass pool
         pool_start(:,:,iele) = pool_start(:,:,iele) + &
              (biomass(:,:,ipar,iele) * veget_max(:,:))
      ENDDO
   ENDDO

   !! 1.2 Initialize variables
   d_mortality(:,:)=zero
   st_mortality(:,:)=zero
   circ_class_mor(:,:,:)=zero
   circ_class_litter(:,:,:)=zero   
   circ_class_leaflitter(:,:,:)=zero
   circ_class_saplitter(:,:,:)=zero
   circ_class_heartlitter(:,:,:)=zero
   kill_st(:,:)=zero
   kill_d(:,:)=zero
   circ_class_st(:,:,:)=zero
 !! 2. Kill PFTs

   ! Kill plants if number of individuals or last year's leaf mass is close to zero.
   ! the "was_killed" business is necessary for a more efficient code on the VPP
   ! processor
   DO ivm = 2,nvm ! loop plant functional types

      was_killed(:) = .FALSE.
!      WRITE(numout,*) 'is_nautral ',natural(ivm),ivm
      IF ( natural(ivm) ) THEN
         
         !! 2.2 Kill natural PFTs when running STOMATE without DGVM

         !! 2.2.1 Kill PFTs 
         !  Kill PFTs but not in the first year because one year
         !  is needed to fill the long-term and seasonal variables.
         !  Therefore no leaves are grown in the first year. As
         !  long as the plant has never grown leaves, ::rue_longterm
         !  keeps its initial value of 1. Note that the logic of
         !  this statement differs from the original verison that
         !  was used for the resource limited allocation where the
         !  carbohydrate reserves pool was treated differently than
         !  in the functional allocation. 
         IF(ld_kill)THEN
            DO ipts=1,npts
               IF( PFTpresent(ipts,ivm) .AND. &
                    (rue_longterm(ipts,ivm) .NE. un) .AND. & 
                    (biomass(ipts,ivm,icarbres,icarbon) .LE. min_stomate .AND. & 
                   biomass(ipts,ivm,ilabile,icarbon) .LE. min_stomate .AND. &
                    biomass(ipts,ivm,ileaf,icarbon) .LE. min_stomate ) .AND. & 
                    ind(ipts,ivm) .GT. zero) THEN
                  
                  WRITE(numout,*) 'Killing PFT, ipts: ',ivm,ipts
                  WRITE(numout,*) 'rue_longterm(ipts,ivm)',rue_longterm(ipts,ivm)
                  WRITE(numout,*) 'biomass(ipts,ivm,icarbres,icarbon)',biomass(ipts,ivm,icarbres,icarbon)
                  WRITE(numout,*) 'biomass(ipts,ivm,ilabile,icarbon)',biomass(ipts,ivm,ilabile,icarbon)
                  WRITE(numout,*) 'biomass(ipts,ivm,ileaf,icarbon) ',biomass(ipts,ivm,ileaf,icarbon) 
                  WRITE(numout,*) 'ind(ipts,ivm)',ind(ipts,ivm)
               ENDIF
            ENDDO
         ENDIF

         WHERE ( PFTpresent(:,ivm) .AND. &
              (rue_longterm(:,ivm) .NE. un) .AND. & 
              (biomass(:,ivm,icarbres,icarbon) .LE. min_stomate .AND. & 
              biomass(:,ivm,ilabile,icarbon) .LE. min_stomate .AND. &
              biomass(:,ivm,ileaf,icarbon) .LE. min_stomate ) .AND. & 
              ind(:,ivm) .GT. zero)

            was_killed(:) = .TRUE.

         ENDWHERE
         
         !++++++++++++++++++++++++++++ testing...REMOVE
!         WRITE(numout,*) 'KILLING test ',ivm,counter
!         IF(counter == 175)THEN
!            WRITE(numout,*) 'KILLING ARTIFICIALLY IN STOMATE_MARK_KILL'
!            was_killed(:)=.TRUE.
!         ENDIF
         !++++++++++++++++++++++++++++

         !! If the density of a forest goes below a certain level,
         !! the forest dies off.  The density limit is referred to
         !! in Bellasen et al 2010.

         IF(is_tree(ivm))THEN

            WHERE ( PFTpresent(:,ivm) .AND. &
               (ind(:,ivm) .LE. dens_target(ivm)*ha_to_m2))
               
               was_killed(:) = .TRUE.
                  
            ENDWHERE

         ENDIF ! checking for trees

         ! Print some information if we are debugging
         IF(ld_kill)THEN
            IF(is_tree(ivm))THEN
               DO ipts=1,npts
                  IF( PFTpresent(ipts,ivm) .AND. &
                       (ind(ipts,ivm) .LE. dens_target(ivm)*ha_to_m2))THEN
                     
                     WRITE(numout,*) 'Killing PFT, ipts: ',ivm,ipts
                     WRITE(numout,*) 'Forest density too low'
                     WRITE(numout,*) 'ind(ipts,ivm)',ind(ipts,ivm)
                     WRITE(numout,*) 'dens_target(ivm)*ha_to_m2',dens_target(ivm)*ha_to_m2
                  ENDIF

               
               ENDDO
            ENDIF

            DO ipts=1,npts
               IF ( PFTpresent(ipts,ivm) .AND. &
                    (rue_longterm(ipts,ivm) .NE. un) .AND. & 
                    (biomass(ipts,ivm,icarbres,icarbon) .LE. min_stomate .AND. & 
                    biomass(ipts,ivm,ilabile,icarbon) .LE. min_stomate .AND. &
                    biomass(ipts,ivm,ileaf,icarbon) .LE. min_stomate ) .AND. & 
                    ind(ipts,ivm) .GT. zero)THEN

                  WRITE(numout,*) 'Killing PFT, ipts: ',ivm,ipts
                  WRITE(numout,*) 'No roots, leaves, labile pool.'
                  WRITE(numout,*) 'ind(ipts,ivm)',ind(ipts,ivm)
                  WRITE(numout,*) 'Reserve pool',biomass(ipts,ivm,icarbres,icarbon)
                  WRITE(numout,*) 'Labile pool',biomass(ipts,ivm,ilabile,icarbon)
                  WRITE(numout,*) 'Leaves',biomass(ipts,ivm,ileaf,icarbon)
                  
               ENDIF
            ENDDO
         ENDIF

         !! 2.3 Bookkeeping for PFTs that were killed
         !  For PFTs that were killed, return biomass pools to litter 
         !  and update biomass pools to zero
         
         DO ipts = 1,npts

            !+++++++++ TEMP +++++++
            !IF(test_pft == ivm)THEN
            !   WRITE(numout,*) 'WAS_KILLED ',was_killed(ipts),rue_longterm(ipts,ivm)
            !ENDIF
            !+++++++++++++++++++++
            !write(*,*) 'was_killed',was_killed(ipts)
            IF ( was_killed(ipts) ) THEN

               !+++++++++ TEMP +++++++
               !IF(test_pft == ivm)THEN
               !   WRITE(numout,*) 'WAS_KILLED KILLED',was_killed(ipts)
               !ENDIF
               !+++++++++++++++++++++

               ! If a PFT is killed, this just means that we mark all the individuals
               ! for a natural death (all debris stays on site).
               ! The killing actually takes place in lpj_gap.f90.
               ! We have to do this differently for trees and non-trees, since
               ! circ_class_n is not defined for non-trees.
               IF(is_tree(ivm))THEN

                  ! In reality, if the forest is managed we will not lose this
                  ! wood.  The forest would be harvested.  So, let's put these into
                  ! different pools depending on the management type.
                  ifm=forest_managed(ipts,ivm)
                  write(*,*) 'forest_managed',ifm

                  DO icir=1,ncirc
                     circ_class_kill(ipts,ivm,icir,ifm,icut_clear)=&
                          circ_class_n(ipts,ivm,icir)
                  ENDDO

                  ! If a stand dies while it is under management, perhaps
                  ! we need to change the species.
                  IF(forest_managed(ipts,ivm) .NE. 1)THEN
                     lpft_replant(ipts,ivm)=.TRUE.
                  ENDIF
                     

               ELSE

                  ! Since lpj_gap only knows about circ_class_kill and circ_class_biomass,
                  ! we need to give values to icir=1 for these variables for grasses
                  ! and crops so that the correct amount of biomass is killed.
                  circ_class_kill(ipts,ivm,1,ifm_none,icut_clear)=ind(ipts,ivm)
                  DO iele=1,nelements
                     DO ipar=1,nparts
                        circ_class_biomass(ipts,ivm,1,ipar,iele)=biomass(ipts,ivm,ipar,iele)/&
                             circ_class_kill(ipts,ivm,1,ifm_none,icut_clear)
                     ENDDO
                  ENDDO

               ENDIF ! is_tree(ivm)

               !! 2.7 Print sub routine messages
               IF ( ld_kill ) THEN

                  WRITE(numout,*) 'kill: eliminated ',PFT_name(ivm)
                  WRITE(numout,*) '  pft number: ',ivm
                  WRITE(numout,*) '  at grid: ',ipts

               ENDIF

            ENDIF ! was_killed

         ENDDO ! loop over points

      ENDIF       ! PFT is natural

      !! We need to enforce two different types of killing at the moment,
      !! self-thinning and environmental mortality.  Self-thinning is due
      !! to resouce competition in dense stands, and environmental
      !! mortality is due to environmental stress.  Self-thinning is
      !! only applicable to forests.  
      DO ipts=1,npts

         ! Don't waste time if the whole PFT was already scheduled to die,
         ! or if the PFT is not here.
         IF(was_killed(ipts))CYCLE
         IF(.NOT. PFTpresent(ipts,ivm))CYCLE
  
         
         !! let's calculate the self-thinning limit first

         ! Number of indiviudals for trees and grasses are calculated differently
         IF ( is_tree(ivm) ) THEN

            ! Calculate the self-thinning density.  This is the target density
            ! for which we will kill small trees until we reach.
            ! there are unit conversions here because the forestry routines
            ! prefer to work with trees per hectare.  I convert everything to
            ! individuals per m**2 when I can.
            Dg=zero
            diameters_temp(:)=wood_to_dia(circ_class_biomass(ipts,ivm,:,:,icarbon),ivm)
            diameters_temp(:)=diameters_temp(:)*m_to_cm

            DO icir=1,ncirc
               Dg=Dg+circ_class_n(ipts,ivm,icir)*m2_to_ha*diameters_temp(icir)**2
            ENDDO

            ! After a coppice at the end of the year, it's possible that all 
            ! the aboveground biomass is killed, but we still have individuals
            ! because the roots are alive.  This means we don't have
            ! a diameter, though, and therefore we will divide by zero
            ! in the next block of code.  So we skip it, which is okay because
            ! there will be no self-thinning in this case anyways.
            IF(Dg .LT. min_stomate) CYCLE

            Dg=SQRT(Dg/(ind(ipts,ivm)*m2_to_ha))
            st_dens(ipts,ivm)=Nmax(Dg,ivm)*ha_to_m2

            IF(ld_kill .AND. ivm == test_pft .AND. ipts == test_grid)THEN
               WRITE(numout,*) 'Does self thinning happen?'
               WRITE(numout,*) 'ipts,ivm: ',ipts,ivm
               WRITE(numout,*) 'ind(ipts,ivm),st_dens(ipts,ivm): ',&
                    ind(ipts,ivm),st_dens(ipts,ivm)
            ENDIF
            
            !write(*,*) 'first Dg',Dg,'ind',ind(ipts,ivm)
            !write(*,*) 'third st_dens',st_dens(ipts,ivm)
            IF(ind(ipts,ivm) .GT. st_dens(ipts,ivm))THEN

               IF(ld_kill .AND. ivm == test_pft)THEN
                  WRITE(numout,*) 'Self thinning is taking place.'
                  WRITE(numout,*) 'ipts,ivm: ',ipts,ivm
                  WRITE(numout,*) 'ind(ipts,ivm),st_dens(ipts,ivm): ',&
                       ind(ipts,ivm),st_dens(ipts,ivm)
                  WRITE(numout,*) 'difference: ',&
                       ind(ipts,ivm)-st_dens(ipts,ivm)
                  WRITE(numout,*) 'st_dist: ',&
                       st_dist(:)
               ENDIF

               ! Here is where we actually do the self thinning.  The number of
               ! trees that we want to kill is the differnce between the
               ! self thinning density and the true density.  Originally, we
               ! killed only small trees until the densities were equal, the
               ! reason being that self-thinning represents competition
               ! driven mortality, and small trees are not able to compete for
               ! resources (e.g. light) as well as larger trees.  However, this
               ! seemed to cause strange results for both temperate and tropical
               ! PFTs, so now we kill trees based on a user-defined distribution.
               difference=ind(ipts,ivm) - st_dens(ipts,ivm)
               !write(*,*) 'ind_diff',difference,'st_dens',st_dens(ipts,ivm),'ind',ind(ipts,ivm) 
               ! This is what we want to kill.
               target_st_kill(:)=st_dist(:)*difference
               ! added by yitong yao in March 22 2020 09:31 cut STmortality 

               !DO icir=1,ncirc
               !   target_st_kill(icir)=target_st_kill(icir) / st_cut_fac
               !ENDDO
               ! added by yitong yao in March 22 2020 09:32 above lines 
               ! However, we may not have a sufficient number of trees in each class,
               ! so we might have to adjust things a bit.
               true_st_kill(:)=target_st_kill(:)

               istep=1
               DO

                  lconverged=.FALSE.

                  ! Find out how many indivuals we cannot kill with the current
                  ! distribution.
                  excess_ind=zero
                  DO icir=1,ncirc
                     IF(circ_class_n(ipts,ivm,icir) .LT. true_st_kill(icir))THEN
                        excess_ind=excess_ind+true_st_kill(icir)-circ_class_n(ipts,ivm,icir)
                        true_st_kill(icir)=circ_class_n(ipts,ivm,icir)
                     ENDIF
                  ENDDO

                  IF(excess_ind .GT. zero)THEN
                     ! We have something to redistribute.
                     ! Put all of the excess in the first class which is not already completely
                     ! killed.  If we add too much, we'll catch it in the next loop.
                     DO icir=1,ncirc
                        IF(circ_class_n(ipts,ivm,icir) .GT. true_st_kill(icir))THEN
                           true_st_kill(icir)=true_st_kill(icir)+excess_ind
                           excess_ind=zero
                        ENDIF
                     ENDDO
                  ELSE
                     ! Done.
                     lconverged=.TRUE.
                  ENDIF

                  IF(lconverged)EXIT

                  istep=istep+1
                  IF(istep .GT. 100)THEN
                     ! This is an arbitrary exit here, just to make sure
                     ! we don't get stuck in an infinite loop.  We should
                     ! exit before we ever get here.  Otherwise, you can increase
                     ! the number in the if statement and try again.
                     WRITE (numout,*) 'ipts, ivm : ',ipts,ivm
                     CALL ipslerr_p (3,'kill_prognostic', &
                          'Was not able to distribute self-thinning individuals!', &
                          'See the comments at this part of the code for a solution','')

                  END IF
               ENDDO
               DO icir=1,ncirc
                  true_st_kill(icir)=true_st_kill(icir) / st_cut_fac
               ENDDO
               circ_class_kill(ipts,ivm,:,ifm_none,icut_thin)=true_st_kill(:)
               !write(*,*) 'circ_class_kill_NONE_thin',circ_class_kill(ipts,ivm,1,ifm_none,icut_thin)
               circ_class_st(ipts,ivm,:)=true_st_kill(:) 
               !DO icir=1,ncirc
               !  write(*,*) 'circ_class_st ',icir,' ',circ_class_st(ipts,ivm,icir)
               !ENDDO
! This is for the old scheme that always took the youngest trees.
!!$               icir=1
!!$               DO
!!$
!!$                  IF(circ_class_n(ipts,ivm,icir) .GT. difference)THEN
!!$                     circ_class_kill(ipts,ivm,icir,ifm_none,icut_thin)=difference
!!$                     difference=zero
!!$                  ELSE
!!$                     circ_class_kill(ipts,ivm,icir,ifm_none,icut_thin)=circ_class_n(ipts,ivm,icir)
!!$                     difference=difference-circ_class_n(ipts,ivm,icir)
!!$                  ENDIF
!!$                  
!!$!                  WRITE(numout,*) 'self thinning difference ',icir,difference,circ_class_n(ipts,ivm,icir),&
!!$!                       ind(ipts,ivm),st_dens(ipts,ivm)
!!$                  IF(difference .LE. min_stomate)EXIT
!!$                  icir=icir+1
!!$                  IF(icir .GT. ncirc)THEN
!!$                     CALL ipslerr(3,'stomate_mark_kill',&
!!$                          'We have a problem.  We are trying to access ',&
!!$                          'circumference classes in ljp_kill that do not exist!','')
!!$                  ENDIF
!!$
!!$               ENDDO
!!$               
               IF(ld_kill .AND. ivm == test_pft)THEN
                  WRITE(numout,*) 'After self thinning.'
                  WRITE(numout,*) 'circ_class_kill: ',circ_class_kill(ipts,ivm,:,ifm_none,icut_thin)
               ENDIF

            ENDIF ! ind(ipts,ivm) .GT. st_dens(ipts,ivm)

            ! now let's look at environmental mortality.  The calculation of this
            ! rate depends on if it is fixed for the whole simulation or if it's
            ! allowed to respond to things like NPP.

            !write(*,*) 'stomate_constant_mortality',control%ok_constant_mortality
            !! 3.1 Use growth efficiency or constant mortality?
            IF ( .NOT. control%ok_constant_mortality  ) THEN

               ! Was the PFT alive last year?
               IF ( ( lm_lastyearmax(ipts,ivm) .GT. min_stomate ) ) THEN

                  !! 3.1.1 Estimate net biomass increment
                  ! Estimate net biomass increment by subtracting turnover from
                  ! last year's NPP.
                  ! Note that npp_longterm is the longterm growth efficiency
                  ! (NPP/LAI) to be fair to deciduous trees
                  delta_biomass = MAX( npp_longterm(ipts,ivm) - &
                       ( turnover_longterm(ipts,ivm,ileaf,icarbon) + &
                       turnover_longterm(ipts,ivm,iroot,icarbon) + &
                       turnover_longterm(ipts,ivm,ifruit,icarbon) + & 
                       turnover_longterm(ipts,ivm,isapabove,icarbon) + &
                       turnover_longterm(ipts,ivm,isapbelow,icarbon) ), zero)

                  !! 3.1.2 Calculate growth efficiency 
                  !  Calculate growth efficiency by dividing net biomass increment
                  !  by last year's maximum LAI. (corresponding actually to the
                  !  maximum LAI of the previous year)
                  vigour = delta_biomass / (lm_lastyearmax(ipts,ivm) * sla(ivm))
                  !write(*,*) 'vigour',vigour
                  ! PFT does not exist or as dead   
               ELSE

                  vigour = zero

               END IF ! ( lm_lastyearmax(ipts,ivm) .GT. min_stomate)

               !! 3.1.3 Calculate growth efficiency mortality rate
               mortality_greff = mortality_max(ivm) / &
                    ( un + ref_mortality(ivm) * vigour )

               ! Scale mortality rate (fraction, 0-1) by timesteps per year 
               mortality = MAX(mortality_min(ivm), mortality_greff) * dt/&
                    one_year  
               write(*,*) 'mortality_vigour',mortality
            ! .NOT. control%ok_constant_mortality
            ELSE

               !! 3.2 Use constant mortality accounting for the residence time 
               !! of each tree PFT

               !  Scale mortality rate (fraction, 0-1) by timesteps per year
               !  NOTE: the meaning of residence_time is very different between
               !  the DOFOCO branch and the trunk. In the trunk biomass has no age
               !  thus the residence time accounts for all forest dynamics including
               !  self-thinning, pests, diseases and wind fall. In the DOFOCO branch
               !  biomass does have an age and self-thinning is explicitly accounted 
               !  for, hence, the residence time should be much higher as it only 
               !  accounts for pest, diseases and windfall. Even the latter is not
               !  exact because as long as those disturbances are small scale they
               !  are probably accounted for in the parametrization of self-thinning.
               mortality = dt/(residence_time(ivm)*one_year)
               write(*,*) 'F_mortality',mortality
!WRITE(*,*) "tzj_smk mortality", mortality
!WRITE(*,*) "tzj_smk restime", residence_time

            ENDIF ! .NOT.  control%ok_constant_mortality
                
           ! added by yitong yao in 06 Jan 2020 22:10  commented in 07 Jan 10:47
           ! DO icir=1,ncirc
           !     IF (stem_plc(1,2,icir) .GE. 50) THEN
           !         mortality=mortality+circ_class_n(1,2,icir)*plc_kill_frac(2)
           !     ENDIF
           ! ENDDO
            ! added by yitong YAO in 06 Jan 2020 22:10 above lines
             
            ! added by yitong yao in 10 Jan 2020 11:13 using linear relaxation
            ! to get smoother estimate for counter 


            ! added by Yitong Yao in 10 Jan 2020 11:14 above lines
             
            ! added by yitong yao in 07 Jan 2020 10:36 
            ! yang hui
            IF (yyt_flag_mortality .EQ. 1) THEN  
             mortality = 0
            ENDIF
             yyt_test=yyt_flag_mortality 
            
            ! yang hui 
            circ_class_P50(:)=zero
            circ_diameters_temp(:)=wood_to_dia(circ_class_biomass(ipts,ivm,:,:,icarbon),ivm)
            CALL P50_vary_class (circ_diameters_temp,circ_class_P50)
            DO icir =1,ncirc
                IF (stem_plc(ipts,ivm,icir) .GE. plc_threshold) THEN 
                   !IF (psi_stem(ipts,ivm,icir) .LT. 0 .AND. psi_stem(ipts,ivm,icir) .GT. -25) THEN
                         !write(*,*) 'psi_stem_',icir,' ',circ_diameters_temp(icir),' ',psi_stem(ipts,ivm,icir),' ', circ_class_P50(icir)
                       counter_psi(ipts,ivm,icir)=counter_psi(ipts,ivm,icir) + 1
                       IF (counter_0(ipts,ivm,icir) .GT. 0) THEN
                          counter_2(ipts,ivm,icir) = counter_2(ipts,ivm,icir) + 1 
                       ELSE
                          counter_2(ipts,ivm,icir) = counter_psi(ipts,ivm,icir)
                       ENDIF
                       counter_0(ipts,ivm,icir) = counter_0(ipts,ivm,icir) + 0
                       ! using flag to judge whether we need relax on
                       ! counter_psi
                        IF (relax_counter .EQ. 1) THEN 
                                IF ((counter_psi(ipts,ivm,icir) .GE. day_threshold) .OR. ((counter_2(ipts,ivm,icir) .GE. day_threshold) .AND. (counter_psi(ipts,ivm,icir) .GT. counter_0(ipts,ivm,icir)) )) THEN
                           !mortality=mortality+circ_class_n(ipts,ivm,icir)*plc_kill_frac(2)
                           circ_class_mor(ipts,ivm,icir)=circ_class_n(ipts,ivm,icir)*mor_kill_frac(2)
                                ENDIF
                        ELSE
                                IF (counter_psi(ipts,ivm,icir) .GE. day_threshold) THEN 
                                     circ_class_mor(ipts,ivm,icir)=circ_class_n(ipts,ivm,icir)*mor_kill_frac(2)
                                ENDIF
                        ENDIF
                       ! flag yitong yao 9 Feb 2021
                   !ENDIF

                ELSE
                    counter_psi(ipts,ivm,icir) = 0
                    IF (counter_2(ipts,ivm,icir) .GT. 0 ) THEN
                        counter_0(ipts,ivm,icir) = counter_0(ipts,ivm,icir) + 1
                    ELSE
                        counter_0(ipts,ivm,icir) = 0
                    ENDIF 
                    counter_2(ipts,ivm,icir)=counter_2(ipts,ivm,icir)+0
                    IF (counter_0(ipts,ivm,icir) .GE. 5) THEN 
                        counter_2(ipts,ivm,icir) = 0
                        counter_0(ipts,ivm,icir) = 0
                    ENDIF
                ENDIF

               ! write(*,*) 'counter_psi ',icir,' ',counter_psi(ipts,ivm,icir)
            ENDDO
            ! added by yitong yao in 07 Jan 2020 10:37 above line 
            !write(*,*) 'day_threshold',day_threshold
            !write(*,*) 'after_mortality',mortality


            !---TEMP---
            IF (ld_kill .AND. ivm .EQ. test_pft) THEN
               WRITE(numout,*) 'Mortality rate (0-1), ', mortality
            ENDIF
            !----------


            ! Now the mortality is just a percentage of the total biomass
            ! on the site.  Since the main variable we output here is
            ! circ_class_kill, we need to convert that biomass into the
            ! number of individuals that are killed, and from which
            ! circumference classes.  Mortality in excess of the self-thinning
            ! mortality will preferenctially kill large trees, since they are
            ! assumed to be less resistant to climatic change.

            ! Aggregate over the different biomass components
            DO ipar = 1,nparts
               IF (yyt_flag_mortality .EQ. 1) THEN
               ! Aggregate over the different circumference classes
                 DO icir = 1,ncirc

                  ! Mortality from self-thinning
                  ! Note that the unit of st_mortaility (g C m-2 dt-1) differs
                  ! from that of mortality (rate dt-1)
                  st_mortality(ipts,ivm) = st_mortality(ipts,ivm) + &
                       circ_class_kill(ipts,ivm,icir,ifm_none,icut_thin) * &
                       circ_class_biomass(ipts,ivm,icir,ipar,icarbon)
                  ! yang hui      
                  d_mortality(ipts,ivm) = d_mortality(ipts,ivm) + & 
                      circ_class_mor(ipts,ivm,icir) * &
                       circ_class_biomass(ipts,ivm,icir,ipar,icarbon) 
                  ! yang hui
                 ENDDO

               ! Ageing and environmental mortality
               ! Note that the unit of d_mortaility (g C m-2 dt-1) differs from 
               ! that of mortality (rate dt-1)
               ! yang hui 
               ! below two lines commented by yitong yao 09 Feb 22:31
                !IF (yyt_flag_mortality .EQ. 0) THEN
              ELSE
                    d_mortality(ipts,ivm) =  d_mortality(ipts,ivm) + &
                    mortality * biomass(ipts,ivm,ipar,icarbon)  
              ENDIF    
               ! yang hui 

            ENDDO ! ipar = 1,nparts
            !write(*,*) 'yyt_st_mortality',st_mortality(ipts,ivm)
            !write(*,*) 'yyt_d_mortality',d_mortality(ipts,ivm)
!WRITE(*,*) "tzj_smk dmortality", d_mortality
!WRITE(*,*) "tzj_smk biomass", biomass(ipts,2,:,icarbon)

            ! added by yitong yao in March 18 2020 23:37

            DO icir=1,ncirc
                IF (circ_class_mor(ipts,ivm,icir) .GT. circ_class_kill(ipts,ivm,icir,ifm_none,icut_thin))  THEN
                    circ_class_kill(ipts,ivm,icir,ifm_none,icut_thin)=circ_class_mor(ipts,ivm,icir)
                    
                ENDIF
            ENDDO
            DO icir=1,ncirc
                write(*,*) 'circ_class_kill', icir,circ_class_kill(ipts,ivm,icir,ifm_none,icut_thin)
                write(*,*) 'circ_class_biomass ', icir, circ_class_biomass(ipts,ivm,icir,7,1), circ_class_biomass(ipts,ivm,icir,4,1)
                IF (circ_class_kill(ipts,ivm,icir,ifm_none,icut_thin) .LT. circ_class_n(ipts,ivm,icir)) THEN
           circ_class_litter(ipts,ivm,icir)=(circ_class_biomass(ipts,ivm,icir,1,1)+circ_class_biomass(ipts,ivm,icir,2,1)+circ_class_biomass(ipts,ivm,icir,4,1)+circ_class_biomass(ipts,ivm,icir,7,1))*circ_class_kill(ipts,ivm,icir,ifm_none,icut_thin)
                
           circ_class_leaflitter(ipts,ivm,icir)=circ_class_biomass(ipts,ivm,icir,1,1)*circ_class_kill(ipts,ivm,icir,ifm_none,icut_thin)   
              circ_class_saplitter(ipts,ivm,icir)=circ_class_biomass(ipts,ivm,icir,2,1)*circ_class_kill(ipts,ivm,icir,ifm_none,icut_thin)   
          circ_class_heartlitter(ipts,ivm,icir)=circ_class_biomass(ipts,ivm,icir,4,1)*circ_class_kill(ipts,ivm,icir,ifm_none,icut_thin)  
        ELSE
         circ_class_litter(ipts,ivm,icir)=(circ_class_biomass(ipts,ivm,icir,1,1)+circ_class_biomass(ipts,ivm,icir,2,1)+circ_class_biomass(ipts,ivm,icir,4,1)+circ_class_biomass(ipts,ivm,icir,7,1))*circ_class_n(ipts,ivm,icir)
         
         circ_class_leaflitter(ipts,ivm,icir)=circ_class_biomass(ipts,ivm,icir,1,1)*circ_class_n(ipts,ivm,icir)
         circ_class_saplitter(ipts,ivm,icir)=circ_class_biomass(ipts,ivm,icir,2,1)*circ_class_n(ipts,ivm,icir)
         circ_class_heartlitter(ipts,ivm,icir)=circ_class_biomass(ipts,ivm,icir,4,1)*circ_class_n(ipts,ivm,icir)
                ENDIF 
            ENDDO
            ! added by yitong yao in March 18 2020 23:37


            IF(ld_kill .AND. test_pft .EQ. ivm .AND. test_grid == ipts)THEN
               WRITE(numout,*) 'Does mortality happen?'
               WRITE(numout,*) 'ipts,ivm: ',ipts,ivm
               WRITE(numout,*) 'd_mortality(ipts,ivm): ',d_mortality(ipts,ivm)
               WRITE(numout,*) 'st_mortality(ipts,ivm): ',st_mortality(ipts,ivm)
            ENDIF

            kill_st(ipts,ivm)=SUM(circ_class_kill(ipts,ivm,:,ifm_none,icut_thin),dim=1)
            kill_d(ipts,ivm)=SUM(circ_class_mor(ipts,ivm,:),dim=1)
            ! do we have extra killing that needs to be done?
            bm_difference=d_mortality(ipts,ivm)- st_mortality(ipts,ivm)

!WRITE(*,*) "tzj_smk bm_diff", bm_difference
            !++++++ CHECK ++++++++++
            ! The question still stands of which trees we kill with the
            ! environmental mortality.  If we only kill the biggest, we
            ! lose a circ class within the first year sometimes.
            IF(bm_difference .GT. zero )THEN

               ! We know the total biomass that we want to kill.  Let's partition 
               ! this biomass among all our circumference classes based on an 
               ! exponential distribution which takes more biomass from
               ! the largest class than the smallest.  There is a chance that we
               ! will not have enough biomass to kill in some classes, in which
               ! case we need to rearrange it a bit.

               IF(ld_kill .AND. ivm == test_pft .AND. ipts == test_grid)THEN
                  WRITE(numout,*) 'Mortality is taking place.'
                  WRITE(numout,*) 'ipts,ivm: ',ipts,ivm
                  WRITE(numout,*) 'bm_difference: ',bm_difference
                  WRITE(numout,*) 'circ_class_kill(ipts,ivm,:,ifm_none,icut_thin): ', &
                    circ_class_kill(ipts,ivm,:,ifm_none,icut_thin)
                   WRITE(numout,*) 'circ_class_n(ipts,ivm,:): ', &
                    circ_class_n(ipts,ivm,:)
               ENDIF


               !CALL distribute_mortality_biomass( bm_difference, &
               !     death_distribution_factor(ivm), circ_class_n(ipts,ivm,:), &
               !     circ_class_biomass(ipts,ivm,:,:,icarbon), &
               !     circ_class_kill(ipts,ivm,:,ifm_none,icut_thin) )
               ! yang hui
               ! commented by yitong yao to replace the routine for
               ! circ_class_kill March 18 2020 23:41
               !CALL distribute_mortality_biomass_yyt( bm_difference, &
               !     death_distribution_factor(ivm), circ_class_n(ipts,ivm,:), &
               !     circ_class_biomass(ipts,ivm,:,:,icarbon), &
               !     circ_class_kill(ipts,ivm,:,ifm_none,icut_thin),circ_class_mor(ipts,ivm,:) )
              !  yang hui 

               IF(ld_kill .AND. ivm == test_pft .AND. ipts == test_grid)THEN
                  WRITE(numout,*) 'after circ_class_kill(ipts,ivm,:,ifm_none,icut_thin): ', &
                    circ_class_kill(ipts,ivm,:,ifm_none,icut_thin)
                  WRITE(numout,*) 'circ_class_n(ipts,ivm,:): ', &
                       circ_class_n(ipts,ivm,:)
               ENDIF

           ENDIF ! bm_difference .GT. zero 

         ELSE ! IF(is_tree(ivm))

            ! we don't seem to use self thinning for grasses, since grasses have
            ! a constant density of 1/m**2.  This is the original code for the
            ! mortality.  Notice that it doesn't actually do anything, due to
            ! Sebastiaan's changes and CHECK statement, so I will leave it 
            ! commented out for now.

            ! one thing I will set is 
            ! circ_class_kill(ipts,ivm,1,ifm_none,icut_thin) and
            ! circ_class_biomass(:,:,1,:,:), since if we decide to
            ! use grass/crop mortality later on we will use these variables
            ! to tell lpj_gap how much biomass to actually kill.
            circ_class_kill(ipts,ivm,1,ifm_none,icut_thin)=zero
            !circ_class_biomass(ipts,ivm,1,:,:)=zero

!!$             IF ( control%ok_dgvm .OR. .NOT.control%ok_constant_mortality) THEN
!!$
!!$                ! +++CHECK+++
!!$                ! For grasses, if last year's NPP is very small (less than 10 gCm^{-2}year{-1})
!!$                ! the grasses completely die. Grasses and crops are deciduous so in the first 
!!$                ! year they have no leaves. Consequently npp_longterm is less then 
!!$                ! ::npp_longterm_init. At the beginning of the second year when rue_longterm is
!!$                ! no longer 1, the PFT is killed immediatly. The IF statement needs to be refined
!!$                ! to properly allow for this type of PFT killing.
!!$                ! The more fundamental and conceptual question is whether we want to kill grasses?
!!$                ! Unless we simulate desertification, it is not clear why we would kill the grass
!!$                ! most likely to replace it by ... grass. Because grass has hardly any biomass it
!!$                ! seem unlikely that killing the grass will affect the C-fluxes.
!!$                ! For both reasons i.e. the technical and conceptual, it was decided NOT to kill
!!$                ! the grass PFTs 
!!$                IF ( PFTpresent(ipts,ivm) .AND. ( npp_longterm(ipts,ivm) .LE. npp_longterm_init ) ) THEN
!!$
!!$                   ! ORIGINAL code - see above to justify the change
!!$                   !!$mortality = un
!!$                   mortality = zero
!!$
!!$                END IF
!!$                ! +++++++++++
!!$
!!$                ! Update biomass and litter pools
!!$                DO ielem = 1,nelements
!!$
!!$                   DO ipart = 1, nparts
!!$
!!$                      IF ( PFTpresent(ipts,ivm) ) THEN
!!$
!!$                         d_mortality(ipts,ivm) =  mortality * biomass(ipts,ivm,ipart,ielem)
!!$                         bm_to_litter(ipts,ivm,ipart,ielem) = bm_to_litter(ipts,ivm,ipart,ielem) + &
!!$                              d_mortality(ipts,ivm)
!!$                         biomass(ipts,ivm,ipart,ielem) = biomass(ipts,ivm,ipart,ielem) - &
!!$                              d_mortality(ipts,ivm)
!!$
!!$                      END IF ! PFTpresent(ipts,ivm)
!!$
!!$                   ENDDO ! nparts
!!$
!!$                END DO ! nelements
!!$
!!$             ENDIF ! .NOT.control%ok_dgvm .AND. .NOT.control%ok_constant_mortality

         ENDIF ! end check for trees

      ENDDO ! loop over grid points

   ENDDO         ! loop over PFTs
   
    !DO icir=1,ncirc
    !    write(*,*) 'circK_class_kill ',icir,' ',sum(sum(circ_class_kill(1,2,icir,:,:),dim=2),dim=1)
    !ENDDO

 !! 4. Check mass balance closure

    !! 4.1 Calculate pools at the end of the routine
    pool_end = zero
    DO ipar = 1,nparts
       DO iele = 1,nelements
          pool_end(:,:,iele) = pool_end(:,:,iele) + &
               (biomass(:,:,ipar,iele) * veget_max(:,:))
       ENDDO
    ENDDO

    !! 4.2 Calculate components of the mass balance
    check_intern(:,:,iatm2land,icarbon) = zero
    check_intern(:,:,iland2atm,icarbon) = -un * zero
    check_intern(:,:,ilat2out,icarbon) = zero
    check_intern(:,:,ilat2in,icarbon) = -un * zero
    check_intern(:,:,ipoolchange,icarbon) = -un * (pool_end(:,:,icarbon) - pool_start(:,:,icarbon))
    closure_intern = zero
    DO imbc = 1,nmbcomp
       closure_intern(:,:,icarbon) = closure_intern(:,:,icarbon) + check_intern(:,:,imbc,icarbon)
    ENDDO

    ! Write outcome
    DO ipts=1,npts
       DO ivm=1,nvm
          IF(ABS(closure_intern(ipts,ivm,icarbon)) .LE. min_stomate)THEN
             IF (ld_massbal) WRITE(numout,*) 'Mass balance closure in kill_prognostic'
          ELSE
             WRITE(numout,*) 'Error: mass balance is not closed in kill_prognostic'
             WRITE(numout,*) '   ipts,ivm; ', ipts,ivm
             WRITE(numout,*) '   Difference is, ', closure_intern(ipts,ivm,icarbon)
             WRITE(numout,*) '   pool_end,pool_start: ', pool_end(ipts,ivm,icarbon), pool_start(ipts,ivm,icarbon)
             IF(ld_stop)THEN
                CALL ipslerr_p (3,'kill_prognostic', 'Mass balance error.','','')
             ENDIF
          ENDIF
       ENDDO
    ENDDO
    
    IF (bavard.GE.2) WRITE(numout,*) 'Leaving kill_prognostic'

 END SUBROUTINE kill_prognostic


END MODULE stomate_mark_kill
